package application

import (
	"encoding/json"
	"fmt"
	"math/rand"
	"net/http"
	"os"
	"path"
	"strconv"
	"time"

	"github.com/google/uuid"
	"sirlab.de/go/knowyt/game"
	"sirlab.de/go/knowyt/log"
	"sirlab.de/go/knowyt/user"
)

type PlayerInfo struct {
	Status   string `json:"status"`
	Authcode string `json:"authcode"`
}

func (app *Application) CreateGame(w http.ResponseWriter, r *http.Request) {
	name := r.URL.Query().Get("name")
	teamname := r.URL.Query().Get("teamname")
	lang := r.URL.Query().Get("lang")
	if lang != "de" && lang != "en" {
		lang = "de"
	}

	authcode := ""
	rnd := rand.New(rand.NewSource(time.Now().UnixNano()))
	for i := 0; i < 6; i++ {
		authcode += strconv.Itoa(rnd.Intn(1000) / 10 % 10)
	}

	gm, err := app.createGame(teamname, lang)
	if err != nil {
		w.WriteHeader(http.StatusInternalServerError)
		log.ErrorLog(err)
		fmt.Fprintf(w, "server error")
		return
	}

	_, err = app.createUser(gm, name, user.ROLE_GAMEMASTER, authcode)
	if err != nil {
		w.WriteHeader(http.StatusInternalServerError)
		log.ErrorLog(err)
		fmt.Fprintf(w, "server error")
		return
	}

	playerInfo := PlayerInfo{
		Status:   "ok",
		Authcode: authcode,
	}
	log.Debug("Name %s, Teamname %s, Authcode %s\n", name, teamname, authcode)

	w.Header().Add("Content-Type", "application/json")
	jsonString, _ := json.Marshal(playerInfo)
	fmt.Fprintf(w, "%s", string(jsonString))
}

func (app *Application) createGame(gamename, lang string) (*game.Game, error) {
	gameId := uuid.NewString()
	gameDirName := path.Join(app.config.DataDir, "games", gameId)
	gameFileName := path.Join(gameDirName, "game.json")
	quotesDirName := path.Join(gameDirName, "quotes")

	if err := os.Mkdir(gameDirName, 0777); err != nil {
		return nil, err
	}
	if err := os.Mkdir(quotesDirName, 0777); err != nil {
		return nil, err
	}
	log.Info("createGame(\"%s\": \"%s\" [%s])\n", gameId, gamename, lang)
	gm, err := game.NewGame(gameId, gameFileName, gamename, lang)
	if err != nil {
		return nil, err
	}

	if err := gm.SaveGame(); err != nil {
		return nil, err
	}

	app.addGame(gm)
	go gm.StartEngine()

	return gm, nil
}
