package game

import (
	"sirlab.de/go/knowyt/log"
	"sirlab.de/go/knowyt/user"
)

func (gm *Game) EventPlayerJoins(usr *user.User) {
	usrId := usr.GetId()
	usrName := usr.GetName()

	gm.mu.Lock()
	defer gm.mu.Unlock()

	prevPlayer := gm.players[usrId]
	player := playerInfo{
		id:        usrId,
		name:      usrName,
		isPlaying: true,
		isIdle:    false,
		score:     prevPlayer.score,
		role:      usr.GetRole(),
	}
	gm.players[usrId] = player

	if prevPlayer.id != "" {
		log.Debug("%s player \"%s\" re-joined\n", gm.id, gm.players[usrId].name)
	} else {
		log.Debug("%s player \"%s\" joined\n", gm.id, usrName)
	}

	gm.eng.Update()
}

func (gm *Game) EventPlayerLeaves(usr *user.User) {
	usrId := usr.GetId()

	gm.mu.Lock()
	defer gm.mu.Unlock()

	player := gm.players[usrId]
	if player.id != usrId {
		return
	}
	player.isIdle = true
	gm.players[usrId] = player

	log.Debug("%s player \"%s\" is idle\n", gm.id, usr.GetName())

	gm.eng.Update()
}

func (gm *Game) AddPlayer(usr *user.User) {
	usrId := usr.GetId()

	gm.mu.Lock()
	defer gm.mu.Unlock()

	player := gm.players[usrId]
	if player.id == usrId {
		return
	}

	gm.players[usrId] = playerInfo{
		id:        usrId,
		name:      usr.GetName(),
		isPlaying: false,
		isIdle:    true,
		score:     0,
		role:      usr.GetRole(),
	}
}

func (gm *Game) UpdatePlayer(usr *user.User) {
	usrId := usr.GetId()

	gm.mu.Lock()
	defer gm.mu.Unlock()

	player := gm.players[usrId]
	if player.id != usrId {
		return
	}

	player.name = usr.GetName()
	gm.players[usrId] = player
}

func (gm *Game) RemovePlayer(usr *user.User) {
	usrId := usr.GetId()

	gm.mu.Lock()
	defer gm.mu.Unlock()
	delete(gm.players, usrId)
}
