package game

import (
	"sirlab.de/go/knowyt/log"
	"sirlab.de/go/knowyt/syncdata"
)

func (gm *Game) populateGetRoundInfo() *syncdata.RoundInfo {
	if gm.state != STATE_PLAY {
		return nil
	}

	quote := gm.quotes[gm.round.quoteId]
	if quote == nil {
		log.Error("%s fatal error: quote id \"%s\" not found\n", gm.id, gm.round.quoteId)
		return nil
	}

	sources := make([]syncdata.SourceInfo, 0)
	for _, source := range gm.round.sources {
		sources = append(sources, syncdata.SourceInfo{
			Id:   source.id,
			Name: source.name,
		})
	}

	return &syncdata.RoundInfo{
		Quote:      quote.GetQuote(),
		Sources:    sources,
		Selections: gm.getSelections(),
		Revelation: gm.getRevelation(),
	}
}

func (gm *Game) getSelections() map[string]bool {
	selections := make(map[string]bool, 0)
	for id, playerInfo := range gm.players {
		if !playerInfo.isPlaying {
			continue
		}

		if len(gm.round.selections[id]) > 0 {
			selections[id] = true
		} else if !playerInfo.isIdle {
			selections[id] = false
		}
	}

	return selections
}

func (gm *Game) getRevelation() *syncdata.Revelation {
	revelation := syncdata.Revelation{
		Votes:   gm.round.revelation.votes,
		Sources: gm.round.revelation.sources,
	}

	return &revelation
}
