package handler

import (
	"net/http"
	"strings"

	"sirlab.de/go/knowyt/application"
	"sirlab.de/go/knowyt/user"
)

type HandlerFunc func(http.ResponseWriter, *http.Request)
type PrivateHandlerFunc func(*user.User, http.ResponseWriter, *http.Request)

type AuthMux struct {
	mux                  *http.ServeMux
	bindInterfaceAndPort string
	app                  *application.Application
}

func NewAuthMux(app *application.Application) *AuthMux {
	bindString := app.GetConfig().BindString
	if !strings.Contains(bindString, ":") {
		bindString = bindString + ":32039"
	}
	mux := AuthMux{
		app:                  app,
		mux:                  http.NewServeMux(),
		bindInterfaceAndPort: bindString,
	}
	http.Handle("/", mux.mux)

	return &mux
}

func (authMux *AuthMux) GetBindString() string {
	if authMux.bindInterfaceAndPort[0] == ':' {
		return "*" + authMux.bindInterfaceAndPort
	}
	return authMux.bindInterfaceAndPort
}

func (authMux *AuthMux) ServeHTTP(w http.ResponseWriter, r *http.Request) {
	authMux.mux.ServeHTTP(w, r)
}

func (authMux *AuthMux) PublicHandleFunc(pattern string, handlerFunc HandlerFunc) {
	authMux.mux.HandleFunc(pattern, handlerFunc)
}

func (authMux *AuthMux) PublicHandle(pattern string, handler http.Handler) {
	authMux.mux.Handle(pattern, handler)
}

func (authMux *AuthMux) ListenAndServe() {
	http.ListenAndServe(authMux.bindInterfaceAndPort, nil)
}
