package handler

import (
	"crypto/rand"
	"fmt"
	"net/http"
	"time"

	"github.com/golang-jwt/jwt"
	"sirlab.de/go/knowyt/user"
)

var secretKey []byte = nil

func (authMux *AuthMux) createToken(uid string) (string, error) {
	if secretKey == nil {
		secretKey = make([]byte, 32)
		if _, err := rand.Read(secretKey); err != nil {
			return "", err
		}
	}

	token := jwt.NewWithClaims(jwt.SigningMethodHS512,
		jwt.MapClaims{
			"uid": uid,
			"exp": time.Now().Add(time.Hour * 24).Unix(),
		})

	return token.SignedString(secretKey)
}

func (authMux *AuthMux) validateSessionAndGetUser(r *http.Request) (*user.User, error) {
	tokenString, err := r.Cookie("knowyt-auth")
	if err != nil {
		return nil, err
	}

	token, err := jwt.Parse(tokenString.Value, func(token *jwt.Token) (interface{}, error) {
		return secretKey, nil
	})
	if err != nil {
		return nil, err
	}

	if !token.Valid {
		return nil, fmt.Errorf("invalid JWT")
	}

	claims, ok := token.Claims.(jwt.MapClaims)
	if !ok {
		return nil, fmt.Errorf("invalid JWT")
	}
	userId := claims["uid"].(string)
	if len(userId) == 0 {
		return nil, fmt.Errorf("invalid JWT")
	}

	usr, err := authMux.app.GetUserById(userId)
	if err != nil {
		return nil, err
	}

	return usr, nil
}
