package main

import (
	"os"

	"sirlab.de/go/knowyt/application"
	"sirlab.de/go/knowyt/applicationConfig"
	"sirlab.de/go/knowyt/handler"
	"sirlab.de/go/knowyt/log"
)

func main() {
	appConfig := applicationConfig.NewApplicationConfig()
	app, err := application.NewApplication(appConfig)
	if err != nil {
		log.ErrorLog(err)
		os.Exit(1)
	}
	mux := handler.NewAuthMux(app)

	mux.PublicHandleFunc("/__intern__/exit", handler.Exit)
	mux.PublicHandleFunc("/api/login", mux.Login)
	mux.PublicHandleFunc("/api/logout", mux.Logout)
	mux.PublicHandleFunc("/api/setupApp", app.SetupApp)
	mux.PublicHandleFunc("/api/createGame", app.CreateGame)
	mux.PrivateOrPublicHandleFunc("/api/userinfo", mux.GetUserInfo, mux.CheckSetup)
	mux.PrivateHandleFunc("/api/cameo", mux.Cameo)
	mux.PrivateHandleFunc("/api/gameinfo", app.GetGameInfo)
	mux.PrivateHandleFunc("/api/games", app.GetGames)
	mux.PrivateHandleFunc("/api/setGameName", app.SetGameName)
	mux.PrivateHandleFunc("/api/setGameLang", app.SetGameLang)
	mux.PrivateHandleFunc("/api/savePlayer", app.SavePlayer)
	mux.PrivateHandleFunc("/api/removePlayer", app.RemovePlayer)
	mux.PrivateHandleFunc("/api/sync", app.SyncHandler)
	mux.PrivateHandleFunc("/api/collectQuotes", app.CollectQuotes)
	mux.PrivateHandleFunc("/api/startGame", app.StartGame)
	mux.PrivateHandleFunc("/api/resetGame", app.ResetGame)
	mux.PrivateHandleFunc("/api/continueGame", app.ContinueGame)
	mux.PrivateHandleFunc("/api/finishGame", app.FinishGame)
	mux.PrivateHandleFunc("/api/disableGame", app.DisableGame)
	mux.PrivateHandleFunc("/api/removeGame", app.RemoveGame)
	mux.PrivateHandleFunc("/api/saveSelection", app.SaveSelection)
	mux.PrivateHandleFunc("/api/getQuotes", app.GetQuotes)
	mux.PrivateHandleFunc("/api/saveQuote", app.SaveQuote)
	mux.PrivateHandleFunc("/api/removeQuote", app.RemoveQuote)

	// default handler
	mux.PublicHandleFunc("/", handler.FileHandler)

	// start listening
	log.Info("Listening at http://%s/\n", mux.GetBindString())
	mux.ListenAndServe()
}
